package com.caimei365.commodity.controller;

import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.vo.MallOrganizeProductVo;
import com.caimei365.commodity.service.MallOrganizeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2023/4/28
 */
@Api(tags = "外部商城")
@RestController
@RequestMapping("/commodity/MallOrganize")
public class MallOrganizeApi {

    @Autowired private MallOrganizeService mallOrganizeService;

    /**
     * 联合丽格小程序首页
     * @return
     */
    @ApiOperation("小程序首页")
    @ApiImplicitParams({
            @ApiImplicitParam(readOnly = true, name = "organizeId", value = "组织Id 1 联合丽格")
    })
    @GetMapping("/mall/init")
    public ResponseJson<Map<String, Object>> getMallHome(Integer organizeId) {
        if (null == organizeId) {
            return ResponseJson.error(-1,"商城类型不能为空",null);
        }
        return mallOrganizeService.getMallHome(organizeId);
    }

    /**
     * 主菜单区详情
     * @param organizeId
     * @param id
     * @return
     */
    @ApiOperation("主菜单区详情")
    @ApiImplicitParams({
            @ApiImplicitParam(readOnly = true, name = "organizeId", value = "组织Id 1 联合丽格"),
            @ApiImplicitParam(readOnly = true, name = "jumpType", value = "跳转类型 1商品详情 2活动列表"),
            @ApiImplicitParam(readOnly = true, name = "id", value = "菜单Id")
    })
    @GetMapping("/mall/detail")
    public ResponseJson<Map<String,Object>> getMenu(Integer organizeId, Integer jumpType, Integer id) {
        if (null == organizeId) {
            return ResponseJson.error(-1,"商城类型不能为空",null);
        }
        if (null == jumpType) {
            return ResponseJson.error(-1,"菜单类型不能为空",null);
        }
        if (null == id) {
            return ResponseJson.error(-1,"菜单id不能为空",null);
        }

        return mallOrganizeService.getMenu(organizeId, jumpType, id);
    }

    /**
     * 商品楼层详情
     * @param activeId
     * @param organizeId
     * @return
     */
    @ApiOperation("商品楼层详情")
    @ApiImplicitParams({
            @ApiImplicitParam(readOnly = true, name = "organizeId", value = "组织Id 1 联合丽格"),
            @ApiImplicitParam(readOnly = true, name = "activeId", value = "活动Id")
    })
    @GetMapping("/mall/organizeProducts")
    public ResponseJson<List<MallOrganizeProductVo>> getOrganizeProducts(Integer organizeId, Integer activeId) {
        if (null == activeId) {
            return ResponseJson.error(-1,"活动id不能为空",null);
        }
        if (null == organizeId) {
            return ResponseJson.error(-1,"商城类型不能为空",null);
        }
        return mallOrganizeService.getOrganizeProducts(activeId, organizeId);
    }
}
