package com.caimei365.commodity.service.impl;

import com.caimei365.commodity.mapper.MallOrganizeMapper;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.vo.*;
import com.caimei365.commodity.service.MallOrganizeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2023/4/28
 */
@Service
public class MallOrganizeServiceImpl implements MallOrganizeService {

    @Autowired private MallOrganizeMapper mallOrganizeMapper;
    /**
     * 联合丽格小程序首页
     *
     * @param organizeId
     * @return
     */
    @Override
    public ResponseJson<Map<String, Object>> getMallHome(Integer organizeId) {
        Map<String, Object> map = new HashMap<>();
        // 头部菜单
        List<MainmenuVo> mainmenu = mallOrganizeMapper.getMainmenu(organizeId);
        map.put("mainmenu", mainmenu);
        // banner图
        List<MallImageLinkVo> bannerImageList = mallOrganizeMapper.getMallBannerImages(organizeId);
        map.put("bannerList", bannerImageList);
        // 专题活动
        List<MallThemeVo> mallThemeVos = mallOrganizeMapper.getmallTheme(organizeId);
        map.put("mallTheme", mallThemeVos);
        // 商品楼层
        List<MallActiveVo> mallActive = mallOrganizeMapper.getMallActive(organizeId);
        // 楼层商品
        for (MallActiveVo active : mallActive) {
            List<MallOrganizeProductVo> mallProducts = mallOrganizeMapper.getMallProducts(active.getId());
            if (null != mallProducts) {
                active.setProducts(mallProducts);
            }
        }
        map.put("mallActive", mallActive);
        return ResponseJson.success(map);
    }

    /**
     * 主菜单区详情
     *
     * @param organizeId
     * @param id
     * @return
     */
    @Override
    public ResponseJson<Map<String,Object>> getMenu(Integer organizeId,Integer jumpType, Integer id) {
        Map<String, Object> map = new HashMap<>();
        // 获取菜单信息
        MainmenuVo menuInfo = mallOrganizeMapper.getMenuInfo(organizeId, id);
        if (menuInfo == null) {
            return ResponseJson.error(-1,"该菜单不存在，请刷新菜单",null);
        }
        if (1 == menuInfo.getJumpType()) {
            // 菜单跳转单商品
            MallOrganizeProductVo menuProductList = mallOrganizeMapper.getMenuProductList(organizeId, id);
            map.put("menuProductList", menuProductList);
        } else {
            // 菜单跳转活动
            if (null != menuInfo.getJumpActiveId()) {
                MallActiveVo active = mallOrganizeMapper.getActiveById(organizeId, menuInfo.getJumpActiveId());
                // 活动宣传图
                List<MallActiveImageVo> imageList = mallOrganizeMapper.getActiveImageById(menuInfo.getJumpActiveId());
                // 活动商品信息
                List<MallOrganizeProductVo> menuList = mallOrganizeMapper.getMenuActiveList(organizeId, id);
                active.setProducts(menuList);
                active.setActiveImageVos(imageList);
                map.put("active", active);
            } else {
                return ResponseJson.error(-1,"该活动不存在，请联系维护人员",null);
            }
        }
        return ResponseJson.success(map);
    }

    /**
     * 商品楼层详情
     *
     * @param activeId
     * @param organizeId
     * @return
     */
    @Override
    public ResponseJson<List<MallOrganizeProductVo>> getOrganizeProducts(Integer activeId, Integer organizeId) {
        List<MallOrganizeProductVo> organizeProducts = mallOrganizeMapper.getOrganizeProducts(activeId, organizeId);
        return ResponseJson.success(organizeProducts);
    }
}
