package com.caimei365.commodity.service.impl;

import com.caimei365.commodity.mapper.ProcureRepository;
import com.caimei365.commodity.model.po.ProcurePo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.LookupOperation;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2023/4/25
 */
@Service
public class ProcureService {

    @Autowired private ProcureRepository procureRepository;
    @Autowired private MongoTemplate mongoTemplate;


    /**
     * 保存
     * @param procure
     */
    public void save(ProcurePo procure){
        //如果需要自定义主键，可以在这里指定主键；如果不指定主键，MongoDB会自动生成主键
        //设置一些默认初始值。。。
        //调用dao
        procureRepository.save(procure);
    }

    /**
     * 修改
     * @param procure
     */
    public void update(ProcurePo procure){
        //调用dao
        procureRepository.save(procure);
    }

    /**
     * 根据id删除
     * @param id
     */
    public void deleteById(String id){
        //调用dao
        procureRepository.deleteById(id);
    }

    /**
     * 查询所有集采
     * @return
     */
    public List<ProcurePo> findList(int pageNo, int pageSize){
        //调用dao
        Query query = new Query();
        query.addCriteria(Criteria.where("delFlag").is("0"));
        query.with(Sort.by(Sort.Direction.DESC, "saveTime"));
        query.limit(pageSize);
        query.skip(pageSize * (pageNo - 1));
        return mongoTemplate.find(query, ProcurePo.class);
    }
    /**
     * 根据id查询
     * @param id
     * @return
     */
    public ProcurePo findById(String id){
        //调用dao
        return procureRepository.findById(id).get();
    }

    /**
     * 我参与的集采
     * @param userId
     * @return
     */
    public List<ProcurePo> findInvolvedList(Integer userId) {
        Query query = new Query();
        query.addCriteria(Criteria.where("userId").is(userId));
        query.addCriteria(Criteria.where("createUserId").not().is(userId));
        query.with(Sort.by(Sort.Direction.DESC, "saveTime"));
        query.with(Sort.by(Sort.Direction.DESC, "isAchieve"));
        List<ProcurePo> procureList = mongoTemplate.find(query, ProcurePo.class);
        procureList.forEach(p -> {

        });
        return mongoTemplate.find(query, ProcurePo.class);
    }

    /**
     * 我发起的集采
     * @param userId
     * @return
     */
    public List<ProcurePo> findUserList(Integer userId) {
        Query query = new Query();
        query.addCriteria(Criteria.where("createUserId").is(userId));
        query.with(Sort.by(Sort.Direction.DESC, "saveTime"));
        query.with(Sort.by(Sort.Direction.DESC, "isAchieve"));
        return mongoTemplate.find(query, ProcurePo.class);
    }

    public ProcurePo editData(Integer id, Integer userId) {
        Query query = new Query();
        query.addCriteria(Criteria.where("id").is(id));
        query.addCriteria(Criteria.where("userId").is(userId));
        return mongoTemplate.find(query, ProcurePo.class).get(0);
    }
}
